// Copyright (c) 2014, The Constrain project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library constraint.metadata;

import 'constraint.dart';
import 'src/preconditions.dart';
import 'src/runtime_constraint_resolver.dart';
import 'package:collection/collection.dart';


abstract class TypeDescriptorResolver {
  factory TypeDescriptorResolver() = RuntimeTypeDescriptorResolver;

  TypeDescriptor resolveFor(Type clazz);
}

// Not sure if we need this or not. Java's one has a fair load of stuff but
// we may be able to stick at least some of them on Constraint itself as Dart's
// annotations are much less sucky
class ConstraintDescriptor<T extends Constraint> {
  final T constraint;

  ConstraintDescriptor(this.constraint) {
    ensure(constraint, isNotNull);
  }
}

abstract class ElementDescriptor {
  final Set<ConstraintDescriptor> constraintDescriptors;

  ElementDescriptor(this.constraintDescriptors) {
    ensure(constraintDescriptors, isNotNull);
  }
}

class MemberDescriptor extends ElementDescriptor {
  final Symbol name;

  MemberDescriptor(this.name, Set<ConstraintDescriptor> constraintDescriptors)
      : super(constraintDescriptors) {
    ensure(name, isNotNull);
  }
}

class TypeDescriptor extends ElementDescriptor {
  final Type type;
  final Map<Symbol, MemberDescriptor> memberDescriptorsMap;
  Iterable<MemberDescriptor> get memberDescriptors => memberDescriptorsMap.values;

  TypeDescriptor(this.type, Set<ConstraintDescriptor> constraintDescriptors,
      Set<MemberDescriptor> memberDescriptors)
      : this.memberDescriptorsMap = _createMap(memberDescriptors),
        super(constraintDescriptors) {
    ensure(type, isNotNull);
    ensure(memberDescriptorsMap, isNotNull);
  }

  static Map<Symbol, MemberDescriptor> _createMap(
      Set<MemberDescriptor> memberDescriptors) {
    ensure(memberDescriptors, isNotNull);
    final map = <Symbol, MemberDescriptor>{};
    memberDescriptors.forEach((m) => map[m.name] = m);
    return new UnmodifiableMapView(map);
  }
}