// Copyright (c) 2014, The Constrain project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library constraint.metadata.resolver.runtime;

import '../constraint.dart';
import 'package:constrain/metadata.dart';
import 'dart:mirrors';


class RuntimeTypeDescriptorResolver implements TypeDescriptorResolver {
  // TODO: Some caching wouldn't hurt but would want to do it with
  // weak references

  @override
  TypeDescriptor resolveFor(Type clazz) {
    final ClassMirror cm = reflectClass(clazz);

    final memberDescriptors = new Set<MemberDescriptor>();

    cm.declarations.forEach((k, v) {
      if (v is VariableMirror) {
        final Set<ConstraintDescriptor> constraintDescriptors = v.metadata
            .where((InstanceMirror im) => im.reflectee is Constraint)
            .map((im) => im.reflectee)
            .map((c) => new ConstraintDescriptor(c))
            .toSet();

        if (constraintDescriptors.isNotEmpty) {
          memberDescriptors.add(new MemberDescriptor(k, constraintDescriptors));
        }
      }
    });

    return new TypeDescriptor(clazz, new Set(), memberDescriptors);
  }
}
