// Copyright (c) 2014, The Constrain project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

import 'package:constrain/constraint.dart';
import 'package:matcher/matcher.dart';
import 'package:constrain/validator.dart';

class Address {
  String street;

  String toString() => 'Address[street: $street]';
}

isNotEmpty() => isNot(isEmpty);

class Person {
  @Ensure(isPositive)
  @Ensure(_isBetween10and90)
  int age;

  @Ensure(isNotNull)
  @Ensure(isNotEmpty)
  @Ensure(_allStreetsStartWith15)
  List<Address> addresses;

  static _allStreetsStartWith15() => everyElement(startsWith("15"));
  static _isBetween10and90() => allOf(greaterThan(10), lessThanOrEqualTo(90));

  String toString() => 'Person[age: $age, addressses: $addresses]';
}


main() {
  final Person person = new Person()
    ..age = 6
    ..addresses = [new Address()..street = "16 blah st"];

  Validator v = new Validator();
  final violations = v.validate(person);
  print(violations);
}


