// Copyright (c) 2014, The Constrain project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library constraint.metadata.test;

import 'package:constrain/metadata.dart';
import 'package:unittest/unittest.dart';
import 'package:constrain/constraint.dart';

main() {
  group('TypeDescriptorResolver', () {
    TypeDescriptorResolver t = new TypeDescriptorResolver();

    group('resolveFor', () {
      group('when class has no constraints should return a TypeDescriptor', () {
        desc() => t.resolveFor(MrNoConstraints);

        test('that is not null', () {
          expect(desc(), isNotNull);
        });

        test('with correct type', () {
          expect(desc().type, equals(MrNoConstraints));
        });

        test('with empty constraint descriptors', () {
          expect(desc().constraintDescriptors, isEmpty);
        });

        test('with empty member descriptors', () {
          expect(desc().memberDescriptors, isEmpty);
        });

        test('with empty member descriptors map', () {
          expect(desc().memberDescriptorsMap, isEmpty);
        });
      });

      group('when class has a member constraint should return a TypeDescriptor', () {
        desc() => t.resolveFor(Person);

        test('that is not null', () {
          expect(desc(), isNotNull);
        });

        test('with correct type', () {
          expect(desc().type, equals(Person));
        });

        test('with empty constraint descriptors', () {
          expect(desc().constraintDescriptors, isEmpty);
        });

        test('with non empty member descriptors', () {
          expect(desc().memberDescriptors, isNot(isEmpty));
        });

        test('with empty member descriptors map', () {
          expect(desc().memberDescriptorsMap, isNot(isEmpty));
        });

        test('with a descriptor on the member', () {
          expect(desc().memberDescriptorsMap[#name], isNotNull);
        });

        test('with one constraint on member', () {
          expect(desc().memberDescriptorsMap[#name].constraintDescriptors,
              hasLength(1));
        });

        test('with expected constraint on member', () {
          expect(desc().memberDescriptorsMap[#name].constraintDescriptors.first.constraint,
              new isInstanceOf<MyMemberConstraint>());
        });
      });
    });
  });
}

class MrNoConstraints {
  String name;
}


class MyMemberConstraint extends Constraint<String> {
  const MyMemberConstraint() : super(null);

  static Set<ConstraintViolation> myValidator(fieldValue, Constraint constraint) {
    return new Set();
  }
}

class Person {
  @MyMemberConstraint()
  String name;
}