// Copyright (c) 2014, The Constrain project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library constraint.validator.test;

import 'package:constrain/validator.dart';
import 'package:unittest/unittest.dart';
import 'package:constrain/constraint.dart';


main() {
  group('validate', () {
    Validator v = new Validator();

    group('should return empty set', () {
      test('when validated object is null', () {
        expect(v.validate(null), isEmpty);
      });

      test('when validated object has no constraints', () {
        expect(v.validate(new MrNoConstraints()..name="fred"), isEmpty);
      });

      test('when field is valid according to constraint', () {
        expect(v.validate(new Person("fred")), isEmpty);
      });
    });

    group('should return non empty set', () {
      final person = new Person(null);

      Set<ConstraintViolation> validate() => v.validate(person);

      test('when field is invalid according to constraint', () {
        expect(validate(), hasLength(1));
      });

      test('with a violation containing the correct constraint', () {
        expect(validate().first.constraintDescriptor.constraint,
            new isInstanceOf<Ensure>());
      });

      test('with a violation containing the correct root', () {
        expect(validate().first.rootObject, equals(person));
      });

      test('with a violation containing the correct leaf', () {
        expect(validate().first.leafObject, equals(person));
      });

      test('with a violation containing the correct path', () {
        expect(validate().first.propertyPath, hasLength(1));
        expect(validate().first.propertyPath, contains(#name));
      });

      test('with a message', () {
        expect(validate().first.message, isNotNull);
      });

      test('with a message starting with correct value', () {
        expect(validate().first.message, startsWith("Invalid Person"));
      });
    });


//
//      group('when class has no constraints should return a TypeDescriptor', () {
//        desc() => t.resolveFor(MrNoConstraints);
//
//        test('that is not null', () {
//          expect(desc(), isNotNull);
//        });
//
//        test('with correct type', () {
//          expect(desc().type, equals(MrNoConstraints));
//        });
//
//        test('with empty constraint descriptors', () {
//          expect(desc().constraintDescriptors, isEmpty);
//        });
//
//        test('with empty member descriptors', () {
//          expect(desc().memberDescriptors, isEmpty);
//        });
//
//        test('with empty member descriptors map', () {
//          expect(desc().memberDescriptorsMap, isEmpty);
//        });
//      });
//
//      group('when class has a member constraint should return a TypeDescriptor', () {
//        desc() => t.resolveFor(Person);
//
//        test('that is not null', () {
//          expect(desc(), isNotNull);
//        });
//
//        test('with correct type', () {
//          expect(desc().type, equals(Person));
//        });
//
//        test('with empty constraint descriptors', () {
//          expect(desc().constraintDescriptors, isEmpty);
//        });
//
//        test('with non empty member descriptors', () {
//          expect(desc().memberDescriptors, isNot(isEmpty));
//        });
//
//        test('with empty member descriptors map', () {
//          expect(desc().memberDescriptorsMap, isNot(isEmpty));
//        });
//
//        test('with a descriptor on the member', () {
//          expect(desc().memberDescriptorsMap[#name], isNotNull);
//        });
//
//        test('with one constraint on member', () {
//          expect(desc().memberDescriptorsMap[#name].constraintDescriptors,
//              hasLength(1));
//        });
//
//        test('with expected constraint on member', () {
//          expect(desc().memberDescriptorsMap[#name].constraintDescriptors.first.constraint,
//              new isInstanceOf<MyMemberConstraint>());
//        });
//      });
//    });
  });
}

class MrNoConstraints {
  String name;
}

Set<ConstraintViolation> _myValidator(fieldValue, Constraint constraint,
    ConstraintValidationContext context) {
  return new Set();
}

class MyMemberConstraint extends Constraint<String> {
  const MyMemberConstraint() : super(_myValidator);

}

class Person {
  @Ensure(isNotNull)
  final String name;

  Person(this.name);

  String toString() => 'Person[name: $name]';
}